/*
 * Copyright 2016 - 2023 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */


#include <nostalgia/core/core.hpp>

#include <teagba/addresses.hpp>
#include <teagba/bios.hpp>
#include <teagba/gfx.hpp>

#include "gfx.hpp"

namespace ox {

using namespace nostalgia::core;

void panic(const char *file, int line, const char *panicMsg, ox::Error const&err) noexcept {
	oxIgnoreError(initGfx(nullptr, {}));
	oxIgnoreError(initConsole(nullptr));
	// enable only BG 0
	REG_DISPCTL = teagba::DispCtl_Bg0;
	clearTileLayer(nullptr, 0);
	ox::BString<23> serr = "Error code: ";
	serr += static_cast<int64_t>(err);
	puts(nullptr, 32 + 1,  1, "SADNESS...");
	puts(nullptr, 32 + 1,  4, "UNEXPECTED STATE:");
	puts(nullptr, 32 + 2,  6, panicMsg);
	if (err) {
		puts(nullptr, 32 + 2,  8, serr);
	}
	puts(nullptr, 32 + 1, 15, "PLEASE RESTART THE SYSTEM");
	// print to terminal if in mGBA
	oxErrf("\033[31;1;1mPANIC:\033[0m [{}:{}]: {}\n", file, line, panicMsg);
	if (err.msg) {
		oxErrf("\tError Message:\t{}\n", err.msg);
	}
	oxErrf("\tError Code:\t{}\n", static_cast<ErrorCode>(err));
	if (err.file != nullptr) {
		oxErrf("\tError Location:\t{}:{}\n", err.file, err.line);
	}
	// disable all interrupt handling and IntrWait on no interrupts
	REG_IE = 0;
	teagba::intrwait(0, 0);
}

}
