/*
 * Copyright 2016 - 2023 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include <ox/std/memory.hpp>

#include <studio/studio.hpp>

#include "paletteeditor-imgui.hpp"
#include "tilesheeteditor-imgui.hpp"

namespace nostalgia::core {

class StudioModule: public studio::Module {
	ox::Vector<studio::EditorMaker> editors(turbine::Context *ctx) const noexcept final {
		return {
			{
				{FileExt_ng},
				[ctx](ox::CRStringView path) -> ox::Result<studio::BaseEditor*> {
					try {
						return ox::make<TileSheetEditorImGui>(ctx, path);
					} catch (const ox::Exception &ex) {
						return ex.toError();
					}
				}
			},
			{
				{FileExt_npal},
				[ctx](ox::CRStringView path) -> ox::Result<studio::BaseEditor*> {
					return PaletteEditorImGui::make(ctx, path);
				}
			}
		};
	}

	ox::Vector<ox::UniquePtr<studio::ItemMaker>> itemMakers(turbine::Context*) const noexcept final {
		ox::Vector<ox::UniquePtr<studio::ItemMaker>> out;
		out.emplace_back(ox::make<studio::ItemMakerT<core::TileSheet>>("Tile Sheet", "TileSheets", "ng"));
		out.emplace_back(ox::make<studio::ItemMakerT<core::Palette>>("Palette", "Palettes", "npal"));
		return out;
	}
};

static StudioModule mod;
const studio::Module *studioModule() noexcept {
	return &mod;
}

}
